package cn.com.qmth.stmms.common.interceptor;

import java.util.Date;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import cn.com.qmth.stmms.admin.utils.SessionExamUtils;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.Marker;
import cn.com.qmth.stmms.biz.exam.model.OperationLog;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.MarkerService;
import cn.com.qmth.stmms.biz.exam.service.OperationLogService;
import cn.com.qmth.stmms.biz.user.model.User;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.domain.ApiUser;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.Role;
import cn.com.qmth.stmms.common.session.model.StmmsSession;
import cn.com.qmth.stmms.common.utils.AccessControlUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;

/**
 * @Description: 日志操作拦截器
 * @author ting.yin
 */
public class LogInterceptor extends HandlerInterceptorAdapter {

    private static Logger log = LoggerFactory.getLogger(LogInterceptor.class);

    @Autowired
    private OperationLogService logService;

    @Autowired
    private ExamService examService;

    @Autowired
    private MarkerService markerService;

    @Autowired
    private UserService userService;

    @Override
    public void postHandle(HttpServletRequest request, javax.servlet.http.HttpServletResponse response, Object handler,
            ModelAndView modelAndView) throws java.lang.Exception {
        WebUser webUser = RequestUtils.getWebUser(request);
        ApiUser apiUser = RequestUtils.getApiUser(request);
        HandlerMethod method = (HandlerMethod) handler;
        String uri = request.getRequestURI();
        if (uri.startsWith("/login")) {
            webUser = buildWebUser(request);
        }
        Logging logging = method.getMethodAnnotation(Logging.class);
        if (webUser != null && logging != null) {
            saveLog(logging, webUser, request);
        }
        if (apiUser != null && logging != null) {
            saveLog(logging, apiUser, request);
        }
    }

    private void saveLog(Logging logging, ApiUser wu, HttpServletRequest request) {
        OperationLog log = new OperationLog();
        log.setCreateTime(new Date());
        log.setMenu(logging.menu());
        log.setType(logging.type());
        log.setMarker(false);
        log.setOperatorId(wu.getUserData().getId());
        log.setIpAddress(wu.getUserData().getLastLoginIp());
        log.setLoginName(wu.getUserData().getLoginName());
        int examId = SessionExamUtils.getExamId(request);
        log.setExamId(examId > 0 ? examId : null);
        log.setSchoolId(wu.getSchoolId());
        log.setDescription(RequestUtils.getLog(request));
        logService.save(log);
    }

    private void saveLog(Logging logging, WebUser wu, HttpServletRequest request) {
        OperationLog log = new OperationLog();
        log.setCreateTime(new Date());
        log.setMenu(logging.menu());
        log.setType(logging.type());
        log.setMarker(wu.isMarker());
        if (wu.isMarker()) {
            log.setMarker(true);
            log.setOperatorId(wu.getMarker().getId());
            log.setIpAddress(wu.getMarker().getLastLoginIp());
            log.setLoginName(wu.getMarker().getLoginName());
            log.setExamId(wu.getMarker().getExamId());
            Exam exam = examService.findById(wu.getMarker().getExamId());
            log.setSchoolId(exam.getSchoolId());
        } else {
            log.setMarker(false);
            log.setOperatorId(wu.getUser().getId());
            log.setIpAddress(wu.getUser().getLastLoginIp());
            log.setLoginName(wu.getUser().getLoginName());
            int examId = SessionExamUtils.getExamId(request);
            log.setExamId(examId > 0 ? examId : null);
            log.setSchoolId(wu.getUser().getSchoolId());
        }
        log.setDescription(RequestUtils.getLog(request));
        logService.save(log);
    }

    /**
     * 从session中构造WebUser对象
     * 
     * @param request
     * @param response
     * @throws Exception
     */
    private WebUser buildWebUser(HttpServletRequest request) {
        StmmsSession session = RequestUtils.getSession(request);
        Integer userId = session.getWebUserId();
        Role role = session.getWebUserRole();
        if (userId != null && role != null) {
            if (Role.MARKER == role) {
                Marker marker = markerService.findById(userId);
                if (marker != null && marker.getAccessTokenRefreshTime() != null
                        && StringUtils.equals(session.getWebUserToken(), marker.getAccessToken())
                        && !AccessControlUtils.expired(marker.getAccessTokenRefreshTime())) {
                    WebUser wu = new WebUser(marker, session.getWebUserLogoutUrl());
                    RequestUtils.setWebUser(request, wu);
                    return wu;
                }
            } else {
                User user = userService.findById(userId);
                if (user != null && user.getRole() == role && user.getAccessTokenRefreshTime() != null
                        && StringUtils.equals(session.getWebUserToken(), user.getAccessToken())
                        && !AccessControlUtils.expired(user.getAccessTokenRefreshTime())) {
                    WebUser wu = new WebUser(user);
                    RequestUtils.setWebUser(request, wu);
                    return wu;
                }
            }
        }
        return null;
    }
}
