package cn.com.qmth.stmms.open.interceptor;

import cn.com.qmth.stmms.biz.user.model.OpenAccount;
import cn.com.qmth.stmms.biz.user.service.UserService;
import cn.com.qmth.stmms.common.domain.OpenUser;
import cn.com.qmth.stmms.common.interceptor.SessionInterceptor;
import cn.com.qmth.stmms.common.session.model.StmmsSession;
import cn.com.qmth.stmms.common.utils.AccessControlUtils;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Mark模块操作拦截器
 *
 * @author XKJ
 */
public class OpenInterceptor extends SessionInterceptor {

    @Autowired
    private UserService userService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        OpenUser ou = buildOpenUser(request);
        if (ou != null) {
            return true;
        }
        return redirect(request, response, "/open/error");
    }

    private OpenUser buildOpenUser(HttpServletRequest request) {
        StmmsSession session = RequestUtils.getSession(request);
        Integer userId = session.getOpenUserId();
        if (userId != null) {
            OpenAccount oa = userService.findOpenAccount(userId);
            if (oa != null && oa.isEnable() && oa.getAccessTokenRefreshTime() != null && StringUtils
                    .equals(session.getOpenUserToken(), oa.getAccessToken()) && !AccessControlUtils
                    .expired(oa.getAccessTokenRefreshTime())) {
                OpenUser ou = new OpenUser(oa, session.getOpenUserReturnUrl());
                RequestUtils.setOpenUser(request, ou);
                return ou;
            }
        }
        return null;
    }
}
